﻿using System.Diagnostics.CodeAnalysis;
using FluentValidation;
using Microsoft.Extensions.DependencyInjection;
using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Services;
using RestApiTemplate.Application.Services.Interfaces;
using RestApiTemplate.Application.Validations;
using RestApiTemplate.Domain.Entities;
using RestApiTemplate.Domain.RepositoriesContracts;
using RestApiTemplate.Infrastructure.Repositories;

namespace RestApiTemplate.CrossCutting.DependencyInjection
{
    /// <summary>
    /// Configurações de injeção de dependência
    /// </summary>
    [ExcludeFromCodeCoverage]
    public static class DependencyInjectionConfiguration
    {
        public static void AddDependencyInjectionConfiguration(this IServiceCollection services)
        {
            services.AddIoCValidators();
            services.AddIoCApplicationServices();
            services.AddIoCRepositories();
        }

        private static void AddIoCValidators(this IServiceCollection services)
        {
            services.AddSingleton<IValidator<ExampleRequest>, ExampleRequestValidation>();
            services.AddSingleton<IValidator<ExampleGetEnabledRequest>, ExampleGetEnableRequestValidation>();
        }

        private static void AddIoCApplicationServices(this IServiceCollection services)
        {
            services.AddScoped<IExampleService, ExampleService>();
            services.AddScoped<IExampleDapperService, ExampleDapperService>();
        }

        private static void AddIoCRepositories(this IServiceCollection services)
        {
            services.AddScoped<IExampleRepository, ExampleRepository>();
        }

    }
}
