﻿using System.Collections.Generic;
using System.Threading.Tasks;

using Rchlo.Omni.Common.Adapter;
using Rchlo.Omni.Common.Extensions;
using Rchlo.Omni.Common.Interfaces;
using Rchlo.Omni.Common.Models;
using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Dtos.Response;
using RestApiTemplate.Application.Services.Interfaces;
using RestApiTemplate.Domain.Entities;
using RestApiTemplate.Domain.RepositoriesContracts;

namespace RestApiTemplate.Application.Services
{
    /// <summary>
    /// Esta classe foi implementada com intuito de realizar consultas complexas, ela não utiliza UnitOfWork 
    /// pois ela somente realiza consultas.
    /// </summary>
    public class ExampleDapperService : IExampleDapperService
    {
        private readonly IAdapter _adapter;
        private readonly IExampleRepository _exampleRepository;

        public ExampleDapperService(IAdapter adapter, IExampleRepository exampleRepository)
        {
            _adapter = adapter;
            _exampleRepository = exampleRepository;
        }

        /// <summary>
        /// É necessário utilizar uma chamada que recupera a quantidade total de registros para se fazer a paginação
        /// </summary>
        /// <param name="example"></param>
        /// <returns></returns>
        public async Task<ReturnMessage<IPagedListCustom<ExampleResponse>>> GetEnabledExamplesAsync(ExampleGetEnabledRequest example)
        {
            var totalCount = await _exampleRepository.CountEnabledExamplesDapperAsync(example.PriceMinimum, example.PriceMaximum)
                .ConfigureAwait(false);

            var examples = await _exampleRepository.GetEnabledExamplesDapperAsync(example.PriceMinimum, example.PriceMaximum, example.PageIndex, example.PageSize)
                 .ConfigureAwait(false);

            var exampleResponse = _adapter.ConvertTo<IEnumerable<Example>, IEnumerable<ExampleResponse>>(examples);

            return new ReturnMessage<IPagedListCustom<ExampleResponse>>(exampleResponse.ToPagedListCustom(example.PageIndex, example.PageSize, totalCount));
        }
    }
}
