﻿using FluentValidation;

using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Domain.Constants;

namespace RestApiTemplate.Application.Validations
{
    /// <summary>
    /// Classe de validações Dto ExampleGetEnabledRequest utilizando FluentValidation
    /// </summary>
    public class ExampleGetEnableRequestValidation : AbstractValidator<ExampleGetEnabledRequest>
    {
        /// <summary>
        /// Exemplos básico de utilização
        /// https://fluentvalidation.net/start#rulesets
        /// </summary>
        public ExampleGetEnableRequestValidation()
        {
            RuleFor(dto => dto)
                   .NotNull();

            RuleFor(dto => dto.PageIndex)
                .GreaterThanOrEqualTo(BussinessConstants.MIN_VALUE_PAGE_INDEX);

            RuleFor(dto => dto.PriceMinimum)
                .GreaterThan(BussinessConstants.PRICE_GREATER_THAN)
                .LessThan(BussinessConstants.PRICE_LESS_THAN);

            RuleFor(dto => dto).Must(_ => _.PriceMinimum < _.PriceMaximum)
                .WithMessage(ErrorsConstants.MIN_VALULE_GREATER_THAN_MAX_VALUE);

            RuleFor(dto => dto.PriceMaximum)
                .LessThan(BussinessConstants.PRICE_LESS_THAN)
                .GreaterThan(BussinessConstants.PRICE_GREATER_THAN);

            RuleFor(dto => dto.PageSize)
                .LessThan(int.MaxValue);

        }
    }
}
