﻿using FluentValidation;

using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Domain.Constants;

namespace RestApiTemplate.Application.Validations
{
    /// <summary>
    /// Classe de validações Dto ExampleRequest utilizando FluentValidation
    /// </summary>
    public class ExampleRequestValidation : AbstractValidator<ExampleRequest>
    {
        /// <summary>
        /// Exemplos básico de utilização
        /// https://fluentvalidation.net/start#rulesets
        /// </summary>
        public ExampleRequestValidation()
        {
            RuleFor(dto => dto)
                .NotNull();

            RuleFor(dto => dto.Name)
                .NotEmpty();

            RuleFor(dto => dto.Price)
                .LessThan(BussinessConstants.PRICE_LESS_THAN)
                .GreaterThan(BussinessConstants.PRICE_GREATER_THAN);

            RuleFor(dto => dto.Password).
                MinimumLength(6).
                MaximumLength(12).WithMessage(ErrorsConstants.PASSWORD_INCORRECT_FORMAT);
        }
    }
}
