﻿using System.Collections.Generic;
using System.Data;
using System.Diagnostics.CodeAnalysis;
using System.Threading.Tasks;
using Rchlo.Omni.Common.Extensions;
using Rchlo.Omni.Infrastructure.Dapper.Repository;
using RestApiTemplate.Domain.Entities;
using RestApiTemplate.Domain.RepositoriesContracts;

namespace RestApiTemplate.Infrastructure.Repositories
{

    /// <summary>
    /// Repostório responsável por fazer as consultas com dapper, utilizando como base DapperRepositoryBase do projeto Rchlo.Omni.Common
    /// Manter o padrão nameof(Objeto.Propriedade).ToSnakeCase() para as queries facilita e agiliza a manutenção.
    /// Exemplo saída nameof(Example.IsEnabled).ToSnakeCase() : "is_enabled"
    /// </summary>
    [ExcludeFromCodeCoverage]
    public class ExampleRepository : DapperRepositoryBase<Example>, IExampleRepository
    {
        public ExampleRepository(IDbConnection connection) : base(connection) { }
        
        public async Task<IEnumerable<Example>> GetEnabledExamplesDapperAsync(decimal priceMinimum, decimal priceMaximum, int pageIndex, int pageSize)
        {
            var query = $"SELECT * FROM { nameof(Example).ToSnakeCase() } Where " +
                                          $" { nameof(Example.Price).ToSnakeCase() }     >= {priceMinimum} and " +
                                          $" { nameof(Example.Price).ToSnakeCase() }     <= {priceMaximum} and " +
                                          $" { nameof(Example.IsEnabled).ToSnakeCase() } = true";
            
            var offset = pageIndex * pageSize;
            
            if (pageSize != 0)
                query += $" Limit {pageSize} Offset {offset}";
            
            return await base.FindAsync(query)
                             .ConfigureAwait(false);
        }
        
        public async Task<int> CountEnabledExamplesDapperAsync(decimal priceMinimum, decimal priceMaximum)
        {
            var query = $"SELECT Count(*) FROM { nameof(Example).ToSnakeCase() } Where " +
                                        $" { nameof(Example.Price).ToSnakeCase() }    >=  {priceMinimum} and " +
                                        $" { nameof(Example.Price).ToSnakeCase() }    <=  {priceMaximum} and " +
                                        $" { nameof(Example.IsEnabled).ToSnakeCase() } = true";
            
            return await base.Count(query).ConfigureAwait(false);
        }
    }
}