﻿using System;
using FakeItEasy;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using Rchlo.Omni.Common.Adapter;
using Rchlo.Omni.Common.Models;
using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Services;
using RestApiTemplate.Domain.RepositoriesContracts;
using Xunit;

namespace RestApiTemplate.Tests.Application.Services
{
    /// <summary>
    /// Classe responsável por realizar os testes unitários do Application-Services
    /// </summary>
    /// <remarks>
    ///     Para nos auxiliar nas implementações dos testes, utilizamos 
    ///     
    ///     FakeItEasy - https://fakeiteasy.readthedocs.io/en/stable/
    /// </remarks>
    public class ExampleDapperServiceTests : BaseTest
    {
        #region Variables
        private readonly ExampleDapperService _exampleService;
        private readonly IUnitOfWork _unitOfWork;
        private readonly IExampleRepository _exampleRepository;
        private readonly IAdapter _adapter;
        private readonly UserLogged _userLogged;
        #endregion

        #region Constructor
        public ExampleDapperServiceTests()
        {
            _unitOfWork = A.Fake<IUnitOfWork>();
            _exampleRepository = A.Fake<IExampleRepository>();
            _adapter = ServiceProvider.GetRequiredService<IAdapter>();
            _userLogged = A.Fake<UserLogged>(_ => _.WithArgumentsForConstructor
            (
                new object[]
                {
                    Guid.NewGuid().ToString(),
                    "User",
                    "user@email",
                    true
                }
            ));

            _exampleService = new ExampleDapperService(_adapter, _exampleRepository);
        }
        #endregion

        #region Tests Methods
        [Fact]
        public void GetEnabledExamplesAsync_WithExistingFilters_ReturnsRecords()
        {
            //Arrange
            var exampleGetEnableRequest = GenerateExampleGetEnabledRequest();

            // Act
            var result = _exampleService.GetEnabledExamplesAsync(exampleGetEnableRequest).Result;

            // Assert
            A.CallTo(() => _exampleRepository.CountEnabledExamplesDapperAsync(A<decimal>.Ignored, A<decimal>.Ignored)).MustHaveHappenedOnceExactly();
            A.CallTo(() => _exampleRepository.GetEnabledExamplesDapperAsync(A<decimal>.Ignored, A<decimal>.Ignored, 
                                                                            A<int>.Ignored, A<int>.Ignored)).MustHaveHappenedOnceExactly();

            Assert.NotNull(result.Data);
            Assert.True(result.Success);
        }

        #endregion

        #region Private Methods
        private ExampleGetEnabledRequest GenerateExampleGetEnabledRequest()
        {
            return new ExampleGetEnabledRequest()
            {
                PageIndex = 0,
                PageSize = 1,
                PriceMaximum = 2,
                PriceMinimum = 1
            };
        }
        #endregion
    }
}
