﻿#region using
using FluentValidation;
using FluentValidation.TestHelper;
using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Validations;
using Xunit;
#endregion

namespace RestApiTemplate.Tests.Application.Validations
{
    /// <summary>
    /// Classe responsável por testar as Validation dos Dto´s
    /// </summary>
    /// <remarks>
    ///     Como os Dto´s são validados de forma automática ([assembly: ApiController] - Startup), 
    ///     temos a necessidade de ter uma classe de testes exclusivas para os Validations
    ///     
    ///     Utilizamos o FluentValidation.TestHelper para auxiliar nos testes
    ///         Link de referência FluentValidation.TestHelper: https://fluentvalidation.net/testing
    /// </remarks>
    public class ExampleRequestValidationTest
    {
        #region Variables
        private readonly IValidator<ExampleRequest> _exampleRequestValidation;
        #endregion

        #region Constructor
        public ExampleRequestValidationTest()
        {
            _exampleRequestValidation = new ExampleRequestValidation();
        }
        #endregion

        #region Test Methods
        [Theory]
        [InlineData(null)]
        [InlineData("")]
        public void ExampleRequestValidation_InvalidNameParameter_ReturnsInvalidNameMessage(string name)
        {
            _exampleRequestValidation.ShouldHaveValidationErrorFor(_ => _.Name, name);
        }

        [Theory]
        [InlineData("Example Name")]
        public void ExampleRequestValidation_ValidNameParameter_ReturnsOk(string name)
        {
            _exampleRequestValidation.ShouldNotHaveValidationErrorFor(_ => _.Name, name);
        }

        [Theory]
        [InlineData(0)]
        [InlineData(11)]
        public void ExampleRequestValidation_InvalidPriceParameter_ReturnsInvalidPriceMessage(decimal price)
        {
            _exampleRequestValidation.ShouldHaveValidationErrorFor(_ => _.Price, price);
        }

        [Theory]
        [InlineData(1)]
        [InlineData(2)]
        public void ExampleRequestValidation_ValidPriceParameter_ReturnsOK(decimal price)
        {
            _exampleRequestValidation.ShouldNotHaveValidationErrorFor(_ => _.Price, price);
        }

        [Theory]
        [InlineData("01234")]
        [InlineData("01234567890123456789")]
        public void ExampleRequestValidation_InvalidPassword_ReturnsInvalidPasswordMessage(string password)
        {
            _exampleRequestValidation.ShouldHaveValidationErrorFor(_ => _.Password, password);
        }

        [Theory]
        [InlineData("0123456")]
        public void ExampleRequestValidation_ValidPassword_ReturnsOk(string password)
        {
            _exampleRequestValidation.ShouldNotHaveValidationErrorFor(_ => _.Password, password);
        }

        #endregion
    }
}
